#! /usr/bin/env python
import pygtk; pygtk.require("2.0")
import gobject
import gconf

## Please take note of this important restriction: all keys in the
## gconf dir being monitored must have a corresponding schema default,
## or else this might break


class GConfSync(gobject.GObject):
    __gsignals__ = {
        'changed': (gobject.SIGNAL_RUN_LAST|gobject.SIGNAL_DETAILED,
                    None, # return type
                    (str, )) # parameter types
    }

    def __init__(self, prefix, enums={}):
        gobject.GObject.__init__(self)
        self.__dict__[" prefix "] = prefix
        gc = gconf.client_get_default()
        keys = dict() # mapping key -> type
        for entry in gc.all_entries(prefix):
            assert entry.key[:len(prefix)] == prefix
            keyname = entry.key[len(prefix):].lstrip('/')
            keytype = gc.get_default_from_schema(entry.key).type
            keys[keyname] = keytype
        self.__dict__["valid keys"] = keys
        self.__dict__["key just been set"] = None
        self.__dict__["enums mapping"] = enums
        reverse_map = dict()
        for key in enums:
            reverse_map[key] = dict(zip(enums[key].values(), enums[key].keys()))
        self.__dict__["enums reverse mapping"] = reverse_map
        gc.add_dir(prefix, gconf.CLIENT_PRELOAD_ONELEVEL)
        gc.notify_add(prefix, self.__gconfclientnotifyfunc)

    def __gconfclientnotifyfunc(self, gc, cnxn_id, entry, user_data):
        prefix = self.__dict__[" prefix "]
        assert entry.key[:len(prefix)] == prefix
        keyname = entry.key[len(prefix):].lstrip('/')
        if keyname and keyname != self.__dict__["key just been set"]:
            self.emit("changed::" + keyname, keyname)
        self.__dict__["key just been set"] = None
        
    def __value_to_python(value):
        if value.type == gconf.VALUE_BOOL:
            if value.get_bool():
                return True
            else:
                return False
        elif value.type == gconf.VALUE_FLOAT:
            return value.get_float()
        elif value.type == gconf.VALUE_INT:
            return value.get_int()
        elif value.type == gconf.VALUE_INVALID:
            raise ValueError, "GConfValue of invalid type"
        elif value.type == gconf.VALUE_LIST:
            return map(GConfSync.value_to_python, value.get_list())
        elif value.type == gconf.VALUE_PAIR:
            raise NotImplementedError
        elif value.type == gconf.VALUE_SCHEMA:
            raise NotImplementedError
        elif value.type == gconf.VALUE_STRING:
            return value.get_string()

    __value_to_python = staticmethod(__value_to_python)

    def __gconf_get(self, key):
        prefix = self.__dict__[" prefix "]
        val = gconf.client_get_default().get("/".join((prefix, key)))
        val = self.__value_to_python(val)
        if key in self.__dict__["enums mapping"]:
            val = self.__dict__["enums mapping"][key][val]
        return val

    def __gconf_set(self, key, val):
        prefix = self.__dict__[" prefix "]
        gc = gconf.client_get_default()
        value_type = self.__dict__["valid keys"][key]
        self.__dict__["key just been set"] = key
        if key in self.__dict__["enums reverse mapping"]:
            val = self.__dict__["enums reverse mapping"][key][val]
        
        key = '/'.join((prefix, key))

        if value_type == gconf.VALUE_BOOL:
            gc.set_bool(key, val)
        elif value_type == gconf.VALUE_FLOAT:
            gc.set_float(key, val)
        elif value_type == gconf.VALUE_INT:
            gc.set_int(key, val)
        elif value_type == gconf.VALUE_INVALID:
            raise ValueError, "GConfValue of invalid type"
        elif value_type == gconf.VALUE_LIST:
            raise NotImplementedError
        elif value_type == gconf.VALUE_PAIR:
            raise NotImplementedError
        elif value_type == gconf.VALUE_SCHEMA:
            raise NotImplementedError
        elif value_type == gconf.VALUE_STRING:
            gc.set_string(key, val)

    def __getattr__(self, key):
        if (key not in self.__dict__["valid keys"]
            or self.__dict__.has_key(key)):
            return self.__dict__[key]
        return self.__gconf_get(key)

    def __setattr__(self, key, val):
        if (key not in self.__dict__["valid keys"]
            or self.__dict__.has_key(key)):
            self.__dict__[key] = val
            return
        self.__gconf_set(key, val)

    def __getitem__(self, key):
        return self.__gconf_get(key)

    def __setitem__(self, key, val):
        self.__gconf_set(key, val)


gobject.type_register(GConfSync)


if __name__ == '__main__':
    import gtk, pango
    prefs = GConfSync("/apps/gnome-osd",
                      dict(osd_halignment=dict(left=pango.ALIGN_LEFT,
                                               right=pango.ALIGN_RIGHT,
                                               center=pango.ALIGN_CENTER)))
    
    def osd_font_changed_cb(obj, key):
        print "'%s' changed" % key
    def osd_halignment_changed_cb(obj, key):
        print "'%s' changed" % key
        print "new value is ", getattr(prefs, key)
        
    prefs.connect("changed::osd_font", osd_font_changed_cb)
    prefs.connect("changed::osd_halignment", osd_halignment_changed_cb)

    print "osd font: ", prefs["osd_font"]
    prefs["osd_font"] = "Foo bar"
    print "osd font (after set): ", prefs.osd_font

    print "osd_halignment is:", prefs.osd_halignment
    print "setting osd_halignment = pango.ALIGN_LEFT"
    prefs.osd_halignment = pango.ALIGN_LEFT
    print "osd_halignment is:", prefs.osd_halignment
    
    gtk.main()

